@@ -1,29 +1,23 @@
 open Names
+open Vmvalues
 open Cbytecodes
 
 type reloc_info =
   | Reloc_annot of annot_switch
   | Reloc_const of structured_constant
-  | Reloc_getglobal of constant
-
-type patch = reloc_info * int
-
-(* A virer *)
-val subst_patch : Mod_subst.substitution -> patch -> patch
+  | Reloc_getglobal of Constant.t
+  | Reloc_proj_name of Projection.Repr.t
 
+type patches
 type emitcodes
 
-val length : emitcodes -> int
+val patch : emitcodes -> patches -> (reloc_info -> int) -> Vmvalues.tcode
 
-val patch_int : emitcodes -> ((*pos*)int * int) list -> emitcodes
-
-type to_patch = emitcodes * (patch list) * fv
-
-val subst_to_patch : Mod_subst.substitution -> to_patch -> to_patch
+type to_patch = emitcodes * patches * fv
 
 type body_code =
   | BCdefined of to_patch
-  | BCalias of constant
+  | BCalias of Constant.t
   | BCconstant
 
 
