@@ -12,7 +12,7 @@ open Names
 open Esubst
 open Constr
 open Declarations
-open Pre_env
+open Environ
 open Nativevalues
 open Nativeinstr
 
@@ -23,7 +23,6 @@ exception NotClosed
 
 type evars =
     { evars_val : existential -> constr option;
-      evars_typ : existential -> types;
       evars_metas : metavariable -> types }
 
 (*s Constructors *)
@@ -102,10 +101,10 @@ let rec map_lam_with_binders g f n lam =
       if body == body' && def == def' then lam else Llet(id,def',body')
   | Lapp(fct,args) ->
       let fct' = f n fct in
-      let args' = Array.smartmap (f n) args in
+      let args' = Array.Smart.map (f n) args in
       if fct == fct' && args == args' then lam else mkLapp fct' args'
   | Lprim(prefix,kn,op,args) ->
-      let args' = Array.smartmap (f n) args in
+      let args' = Array.Smart.map (f n) args in
       if args == args' then lam else Lprim(prefix,kn,op,args')
   | Lcase(annot,t,a,br) ->
       let t' = f n t in
@@ -116,7 +115,7 @@ let rec map_lam_with_binders g f n lam =
 	  if Array.is_empty ids then f n body 
 	  else f (g (Array.length ids) n) body in
 	if body == body' then b else (cn,ids,body') in
-      let br' = Array.smartmap on_b br in
+      let br' = Array.Smart.map on_b br in
       if t == t' && a == a' && br == br' then lam else Lcase(annot,t',a',br')
   | Lif(t,bt,bf) ->
       let t' = f n t in
@@ -124,31 +123,30 @@ let rec map_lam_with_binders g f n lam =
       let bf' = f n bf in
       if t == t' && bt == bt' && bf == bf' then lam else Lif(t',bt',bf')
   | Lfix(init,(ids,ltypes,lbodies)) ->
-      let ltypes' = Array.smartmap (f n) ltypes in
-      let lbodies' = Array.smartmap (f (g (Array.length ids) n)) lbodies in
+      let ltypes' = Array.Smart.map (f n) ltypes in
+      let lbodies' = Array.Smart.map (f (g (Array.length ids) n)) lbodies in
       if ltypes == ltypes' && lbodies == lbodies' then lam
       else Lfix(init,(ids,ltypes',lbodies'))
   | Lcofix(init,(ids,ltypes,lbodies)) ->
-      let ltypes' = Array.smartmap (f n) ltypes in
-      let lbodies' = Array.smartmap (f (g (Array.length ids) n)) lbodies in
+      let ltypes' = Array.Smart.map (f n) ltypes in
+      let lbodies' = Array.Smart.map (f (g (Array.length ids) n)) lbodies in
       if ltypes == ltypes' && lbodies == lbodies' then lam
       else Lcofix(init,(ids,ltypes',lbodies'))
   | Lmakeblock(prefix,cn,tag,args) ->
-      let args' = Array.smartmap (f n) args in
+      let args' = Array.Smart.map (f n) args in
       if args == args' then lam else Lmakeblock(prefix,cn,tag,args')
   | Luint u ->
     let u' = map_uint g f n u in
     if u == u' then lam else Luint u'
-  | Levar (evk, ty, args) ->
-    let ty' = f n ty in
-    let args' = Array.smartmap (f n) args in
-    if ty == ty' && args == args' then lam else Levar (evk, ty', args')
+  | Levar (evk, args) ->
+    let args' = Array.Smart.map (f n) args in
+    if args == args' then lam else Levar (evk, args')
 
 and map_uint g f n u =
   match u with
   | UintVal _ -> u
   | UintDigits(prefix,c,args) ->
-    let args' = Array.smartmap (f n) args in
+    let args' = Array.Smart.map (f n) args in
     if args == args' then u else UintDigits(prefix,c,args')
   | UintDecomp(prefix,c,a) ->
     let a' = f n a in
@@ -181,7 +179,7 @@ let rec lam_exsubst subst lam =
 
 let lam_subst_args subst args =
   if is_subs_id subst then args 
-  else Array.smartmap (lam_exsubst subst) args
+  else Array.Smart.map (lam_exsubst subst) args
   
 (** Simplification of lambda expression *)
       
@@ -276,7 +274,7 @@ and simplify_app substf f substa args =
   (* TODO | Lproj -> simplify if the argument is known or a known global *)
   | _ -> mkLapp (simplify substf f) (simplify_args substa args)
   
-and simplify_args subst args = Array.smartmap (simplify subst) args
+and simplify_args subst args = Array.Smart.map (simplify subst) args
 
 and reduce_lapp substf lids body substa largs =
   match lids, largs with
@@ -337,54 +335,13 @@ let rec get_alias env (kn, u as p) =
 
 (*i Global environment *)
 
-let global_env = ref empty_env 
-
-let set_global_env env = global_env := env
-
 let get_names decl = 
   let decl = Array.of_list decl in
   Array.map fst decl
 
-(* Rel Environment *)
-module Vect = 
-  struct
-    type 'a t = {
-	mutable elems : 'a array;
-	mutable size : int;
-      }
-
-    let make n a = {
-      elems = Array.make n a;
-      size = 0;
-    }
-
-    let extend v =
-      if Int.equal v.size (Array.length v.elems) then 
-	let new_size = min (2*v.size) Sys.max_array_length in
-	if new_size <= v.size then invalid_arg "Vect.extend";
-	let new_elems = Array.make new_size v.elems.(0) in
-	Array.blit v.elems 0 new_elems 0 (v.size);
-	v.elems <- new_elems
-
-    let push v a = 
-      extend v;
-      v.elems.(v.size) <- a;
-      v.size <- v.size + 1
-
-    let popn v n =
-      v.size <- max 0 (v.size - n)
-
-    let pop v = popn v 1
-	
-    let get_last v n =
-      if v.size <= n then invalid_arg "Vect.get:index out of bounds";
-      v.elems.(v.size - n - 1)
-
-  end
-
 let empty_args = [||]
 
-module Renv = 
+module Cache =
   struct
 
     module ConstrHash =
@@ -398,46 +355,20 @@ module Renv =
 
     type constructor_info = tag * int * int (* nparam nrealargs *)
 
-    type t = {
-	name_rel : Name.t Vect.t;
-	construct_tbl : constructor_info ConstrTable.t;
-
-       }
-
-
-    let make () = {
-      name_rel = Vect.make 16 Anonymous;
-      construct_tbl = ConstrTable.create 111
-    }
-
-    let push_rel env id = Vect.push env.name_rel id
-
-    let push_rels env ids = 
-      Array.iter (push_rel env) ids
-
-    let pop env = Vect.pop env.name_rel
-	    
-    let popn env n =
-      for _i = 1 to n do pop env done
-
-    let get env n =
-      Lrel (Vect.get_last env.name_rel (n-1), n)
-
-    let get_construct_info env c =
-      try ConstrTable.find env.construct_tbl c
+    let get_construct_info cache env c : constructor_info =
+      try ConstrTable.find cache c
       with Not_found -> 
 	let ((mind,j), i) = c in
-	let oib = lookup_mind mind !global_env in
+        let oib = lookup_mind mind env in
 	let oip = oib.mind_packets.(j) in
 	let tag,arity = oip.mind_reloc_tbl.(i-1) in
 	let nparams = oib.mind_nparams in
 	let r = (tag, nparams, arity) in
-	ConstrTable.add env.construct_tbl c r;
+        ConstrTable.add cache c r;
 	r
   end
 
-(* What about pattern matching ?*)
-let is_lazy prefix t =
+let is_lazy env prefix t =
   match kind t with
   | App (f,args) ->
      begin match kind f with
@@ -445,94 +376,103 @@ let is_lazy prefix t =
 	let entry = mkInd (fst c) in
 	(try
 	    let _ =
-	      Retroknowledge.get_native_before_match_info (!global_env).retroknowledge
+              Retroknowledge.get_native_before_match_info env.retroknowledge
 							  entry prefix c Llazy;
 	    in
 	    false
 	  with Not_found -> true)
      | _ -> true
      end
-  | LetIn _ -> true
+  | LetIn _ | Case _ | Proj _ -> true
   | _ -> false
 
 let evar_value sigma ev = sigma.evars_val ev
 
-let evar_type sigma ev = sigma.evars_typ ev
-
 let meta_type sigma mv = sigma.evars_metas mv
 
 let empty_evars =
   { evars_val = (fun _ -> None);
-    evars_typ = (fun _ -> assert false);
     evars_metas = (fun _ -> assert false) }
 
 let empty_ids = [||]
 
-let rec lambda_of_constr env sigma c =
+(** Extract the inductive type over which a fixpoint is decreasing *)
+let rec get_fix_struct env i t = match kind (Reduction.whd_all env t) with
+| Prod (na, dom, t) ->
+  if Int.equal i 0 then
+    let dom = Reduction.whd_all env dom in
+    let (dom, _) = decompose_appvect dom in
+    match kind dom with
+    | Ind (ind, _) -> ind
+    | _ -> assert false
+  else
+    let env = Environ.push_rel (RelDecl.LocalAssum (na, dom)) env in
+    get_fix_struct env (i - 1) t
+| _ -> assert false
+
+let rec lambda_of_constr cache env sigma c =
   match kind c with
   | Meta mv ->
      let ty = meta_type sigma mv in
-     Lmeta (mv, lambda_of_constr env sigma ty)
+     Lmeta (mv, lambda_of_constr cache env sigma ty)
 
   | Evar (evk,args as ev) ->
      (match evar_value sigma ev with
      | None ->
-        let ty = evar_type sigma ev in
-        let args = Array.map (lambda_of_constr env sigma) args in
-        Levar(evk, lambda_of_constr env sigma ty, args)
-     | Some t -> lambda_of_constr env sigma t)
+        let args = Array.map (lambda_of_constr cache env sigma) args in
+        Levar(evk, args)
+     | Some t -> lambda_of_constr cache env sigma t)
 
-  | Cast (c, _, _) -> lambda_of_constr env sigma c
+  | Cast (c, _, _) -> lambda_of_constr cache env sigma c
 
-  | Rel i -> Renv.get env i
+  | Rel i -> Lrel (RelDecl.get_name (Environ.lookup_rel i env), i)
 
   | Var id -> Lvar id
 
   | Sort s -> Lsort s
 
   | Ind (ind,u as pind) ->
-      let prefix = get_mind_prefix !global_env (fst ind) in
+      let prefix = get_mind_prefix env (fst ind) in
       Lind (prefix, pind)
 
   | Prod(id, dom, codom) ->
-      let ld = lambda_of_constr env sigma dom in
-      Renv.push_rel env id;
-      let lc = lambda_of_constr env sigma codom in
-      Renv.pop env;
+      let ld = lambda_of_constr cache env sigma dom in
+      let env = Environ.push_rel (RelDecl.LocalAssum (id, dom)) env in
+      let lc = lambda_of_constr cache env sigma codom in
       Lprod(ld,  Llam([|id|], lc))
 
   | Lambda _ ->
       let params, body = Term.decompose_lam c in
+      let fold (na, t) env = Environ.push_rel (RelDecl.LocalAssum (na, t)) env in
+      let env = List.fold_right fold params env in
+      let lb = lambda_of_constr cache env sigma body in
       let ids = get_names (List.rev params) in
-      Renv.push_rels env ids;
-      let lb = lambda_of_constr env sigma body in
-      Renv.popn env (Array.length ids);
       mkLlam ids lb
 
-  | LetIn(id, def, _, body) ->
-      let ld = lambda_of_constr env sigma def in
-      Renv.push_rel env id;
-      let lb = lambda_of_constr env sigma body in
-      Renv.pop env;
+  | LetIn(id, def, t, body) ->
+      let ld = lambda_of_constr cache env sigma def in
+      let env = Environ.push_rel (RelDecl.LocalDef (id, def, t)) env in
+      let lb = lambda_of_constr cache env sigma body in
       Llet(id, ld, lb)
 
-  | App(f, args) -> lambda_of_app env sigma f args
+  | App(f, args) -> lambda_of_app cache env sigma f args
 
-  | Const _ -> lambda_of_app env sigma c empty_args
+  | Const _ -> lambda_of_app cache env sigma c empty_args
 
-  | Construct _ ->  lambda_of_app env sigma c empty_args
+  | Construct _ ->  lambda_of_app cache env sigma c empty_args
 
   | Proj (p, c) ->
-    let kn = Projection.constant p in
-      mkLapp (Lproj (get_const_prefix !global_env kn, kn)) [|lambda_of_constr env sigma c|]
+    let ind = Projection.inductive p in
+    let prefix = get_mind_prefix env (fst ind) in
+    mkLapp (Lproj (prefix, ind, Projection.arg p)) [|lambda_of_constr cache env sigma c|]
 
   | Case(ci,t,a,branches) ->  
       let (mind,i as ind) = ci.ci_ind in
-      let mib = lookup_mind mind !global_env in
+      let mib = lookup_mind mind env in
       let oib = mib.mind_packets.(i) in
       let tbl = oib.mind_reloc_tbl in 
       (* Building info *)
-      let prefix = get_mind_prefix !global_env mind in
+      let prefix = get_mind_prefix env mind in
       let annot_sw = 
 	    { asw_ind = ind;
           asw_ci = ci;
@@ -541,21 +481,21 @@ let rec lambda_of_constr env sigma c =
           asw_prefix = prefix}
       in
       (* translation of the argument *)
-      let la = lambda_of_constr env sigma a in
+      let la = lambda_of_constr cache env sigma a in
       let entry = mkInd ind in
       let la =
 	try
-	  Retroknowledge.get_native_before_match_info (!global_env).retroknowledge
+          Retroknowledge.get_native_before_match_info (env).retroknowledge
 						      entry prefix (ind,1) la
 	with Not_found -> la
       in
       (* translation of the type *)
-      let lt = lambda_of_constr env sigma t in
+      let lt = lambda_of_constr cache env sigma t in
       (* translation of branches *)
       let mk_branch i b =
 	let cn = (ind,i+1) in
 	let _, arity = tbl.(i) in
-	let b = lambda_of_constr env sigma b in
+        let b = lambda_of_constr cache env sigma b in
 	if Int.equal arity 0 then (cn, empty_ids, b)
 	else 
 	  match b with
@@ -568,86 +508,90 @@ let rec lambda_of_constr env sigma c =
       let bs = Array.mapi mk_branch branches in
       Lcase(annot_sw, lt, la, bs)
 	
-  | Fix(rec_init,(names,type_bodies,rec_bodies)) ->
-      let ltypes = lambda_of_args env sigma 0 type_bodies in
-      Renv.push_rels env names;
-      let lbodies = lambda_of_args env sigma 0 rec_bodies in
-      Renv.popn env (Array.length names);
-      Lfix(rec_init, (names, ltypes, lbodies))
+  | Fix((pos, i), (names,type_bodies,rec_bodies)) ->
+      let ltypes = lambda_of_args cache env sigma 0 type_bodies in
+      let map i t =
+        let ind = get_fix_struct env i t in
+        let prefix = get_mind_prefix env (fst ind) in
+        (prefix, ind)
+      in
+      let inds = Array.map2 map pos type_bodies in
+      let env = Environ.push_rec_types (names, type_bodies, rec_bodies) env in
+      let lbodies = lambda_of_args cache env sigma 0 rec_bodies in
+      Lfix((pos, inds, i), (names, ltypes, lbodies))
 	
   | CoFix(init,(names,type_bodies,rec_bodies)) ->
-      let rec_bodies = Array.map2 (Hook.get Clambda.eta_expand !global_env) rec_bodies type_bodies in
-      let ltypes = lambda_of_args env sigma 0 type_bodies in 
-      Renv.push_rels env names;
-      let lbodies = lambda_of_args env sigma 0 rec_bodies in
-      Renv.popn env (Array.length names);
+      let rec_bodies = Array.map2 (Reduction.eta_expand env) rec_bodies type_bodies in
+      let ltypes = lambda_of_args cache env sigma 0 type_bodies in
+      let env = Environ.push_rec_types (names, type_bodies, rec_bodies) env in
+      let lbodies = lambda_of_args cache env sigma 0 rec_bodies in
       Lcofix(init, (names, ltypes, lbodies))
 
-and lambda_of_app env sigma f args =
+and lambda_of_app cache env sigma f args =
   match kind f with
   | Const (kn,u as c) ->
-      let kn,u = get_alias !global_env c in
-      let cb = lookup_constant kn !global_env in
+      let kn,u = get_alias env c in
+      let cb = lookup_constant kn env in
       (try
-          let prefix = get_const_prefix !global_env kn in
+          let prefix = get_const_prefix env kn in
 	  (* We delay the compilation of arguments to avoid an exponential behavior *)
 	  let f = Retroknowledge.get_native_compiling_info
-		    (!global_env).retroknowledge (mkConst kn) prefix in
-	  let args = lambda_of_args env sigma 0 args in
+                    (env).retroknowledge (mkConst kn) prefix in
+          let args = lambda_of_args cache env sigma 0 args in
 	  f args
       with Not_found ->
       begin match cb.const_body with
       | Def csubst -> (* TODO optimize if f is a proj and argument is known *)
           if cb.const_inline_code then
-            lambda_of_app env sigma (Mod_subst.force_constr csubst) args
+            lambda_of_app cache env sigma (Mod_subst.force_constr csubst) args
           else
-          let prefix = get_const_prefix !global_env kn in
+          let prefix = get_const_prefix env kn in
           let t =
-            if is_lazy prefix (Mod_subst.force_constr csubst) then
+            if is_lazy env prefix (Mod_subst.force_constr csubst) then
               mkLapp Lforce [|Lconst (prefix, (kn,u))|]
             else Lconst (prefix, (kn,u))
           in
-        mkLapp t (lambda_of_args env sigma 0 args)
+        mkLapp t (lambda_of_args cache env sigma 0 args)
       | OpaqueDef _ | Undef _ ->
-          let prefix = get_const_prefix !global_env kn in
-          mkLapp (Lconst (prefix, (kn,u))) (lambda_of_args env sigma 0 args)
+          let prefix = get_const_prefix env kn in
+          mkLapp (Lconst (prefix, (kn,u))) (lambda_of_args cache env sigma 0 args)
       end)
   | Construct (c,u) ->
-      let tag, nparams, arity = Renv.get_construct_info env c in
+      let tag, nparams, arity = Cache.get_construct_info cache env c in
       let expected = nparams + arity in
       let nargs = Array.length args in
-      let prefix = get_mind_prefix !global_env (fst (fst c)) in
+      let prefix = get_mind_prefix env (fst (fst c)) in
       if Int.equal nargs expected then 
       try
 	try
 	  Retroknowledge.get_native_constant_static_info
-                         (!global_env).retroknowledge
+                         (env).retroknowledge
                          f args
           with NotClosed ->
 	    assert (Int.equal nparams 0); (* should be fine for int31 *)
-	    let args = lambda_of_args env sigma nparams args in
+            let args = lambda_of_args cache env sigma nparams args in
 	    Retroknowledge.get_native_constant_dynamic_info
-                           (!global_env).retroknowledge f prefix c args
+                           (env).retroknowledge f prefix c args
         with Not_found ->
-	  let args = lambda_of_args env sigma nparams args in
-	  makeblock !global_env c u tag args
+          let args = lambda_of_args cache env sigma nparams args in
+          makeblock env c u tag args
       else
-	let args = lambda_of_args env sigma 0 args in
+        let args = lambda_of_args cache env sigma 0 args in
 	(try
 	    Retroknowledge.get_native_constant_dynamic_info
-              (!global_env).retroknowledge f prefix c args
+              (env).retroknowledge f prefix c args
 	  with Not_found ->
             mkLapp (Lconstruct (prefix, (c,u))) args)
   | _ -> 
-      let f = lambda_of_constr env sigma f in
-      let args = lambda_of_args env sigma 0 args in
+      let f = lambda_of_constr cache env sigma f in
+      let args = lambda_of_args cache env sigma 0 args in
       mkLapp f args
 	
-and lambda_of_args env sigma start args =
+and lambda_of_args cache env sigma start args =
   let nargs = Array.length args in
   if start < nargs then
     Array.init (nargs - start) 
-      (fun i -> lambda_of_constr env sigma args.(start + i))
+      (fun i -> lambda_of_constr cache env sigma args.(start + i))
   else empty_args
 
 let optimize lam =
@@ -660,11 +604,8 @@ let optimize lam =
   lam
 
 let lambda_of_constr env sigma c =
-  set_global_env env;
-  let env = Renv.make () in
-  let ids = List.rev_map RelDecl.get_name !global_env.env_rel_context.env_rel_ctx in
-  Renv.push_rels env (Array.of_list ids);
-  let lam = lambda_of_constr env sigma c in
+  let cache = Cache.ConstrTable.create 91 in
+  let lam = lambda_of_constr cache env sigma c in
 (*  if Flags.vm_draw_opt () then begin
     (msgerrnl (str "Constr = \n" ++ pr_constr c);flush_all()); 
     (msgerrnl (str "Lambda = \n" ++ pp_lam lam);flush_all()); 
