@@ -10,7 +10,6 @@
 
 open Names
 open Declarations
-open Environ
 open Mod_subst
 open Modops
 open Nativecode
@@ -30,15 +29,15 @@ and translate_field prefix mp env acc (l,x) =
   | SFBconst cb ->
      let con = Constant.make3 mp DirPath.empty l in
      (if !Flags.debug then
-	let msg = Printf.sprintf "Compiling constant %s..." (Constant.to_string con) in
+        let msg = Printf.sprintf "Compiling constant %s..." (Constant.to_string con) in
 	Feedback.msg_debug (Pp.str msg));
-     compile_constant_field (pre_env env) prefix con acc cb
+     compile_constant_field env prefix con acc cb
   | SFBmind mb ->
      (if !Flags.debug then
 	let id = mb.mind_packets.(0).mind_typename in
 	let msg = Printf.sprintf "Compiling inductive %s..." (Id.to_string id) in
 	Feedback.msg_debug (Pp.str msg));
-     compile_mind_field prefix mp l acc mb
+     compile_mind_field mp l acc mb
   | SFBmodule md ->
      let mp = md.mod_mp in
      (if !Flags.debug then
