@@ -1,16 +1,17 @@
 (************************************************************************)
-(*  v      *   The Coq Proof Assistant  /  The Coq Development Team     *)
-(* <O___,, *   INRIA - CNRS - LIX - LRI - PPS - Copyright 1999-2016     *)
+(*         *   The Coq Proof Assistant / The Coq Development Team       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
-(*    //   *      This file is distributed under the terms of the       *)
-(*         *       GNU Lesser General Public License Version 2.1        *)
+(*    //   *    This file is distributed under the terms of the         *)
+(*         *     GNU Lesser General Public License Version 2.1          *)
+(*         *     (see LICENSE file for the text of the license)         *)
 (************************************************************************)
 
 open Libnames
 open Pp
-open Util
 
-module Dyn = Dyn.Make(struct end)
+module Dyn = Dyn.Make ()
 
 type 'a substitutivity =
     Dispose | Substitute of 'a | Keep of 'a | Anticipate of 'a
@@ -91,21 +92,13 @@ let declare_object_full odecl =
 			     dyn_rebuild_function = rebuild };
   (infun,outfun)
 
-(* The "try .. with .. " allows for correct printing when calling
-   declare_object a loading time.
-*)
-
-let declare_object odecl =
-  try fst (declare_object_full odecl)
-  with e -> CErrors.fatal_error (CErrors.print e) (CErrors.is_anomaly e)
-let declare_object_full odecl =
-  try declare_object_full odecl
-  with e -> CErrors.fatal_error (CErrors.print e) (CErrors.is_anomaly e)
+let declare_object odecl = fst (declare_object_full odecl)
+let declare_object_full odecl = declare_object_full odecl
 
 (* this function describes how the cache, load, open, and export functions
    are triggered. *)
 
-let apply_dyn_fun deflt f lobj =
+let apply_dyn_fun f lobj =
   let tag = object_tag lobj in
   let dodecl =
     try Hashtbl.find cache_tab tag
@@ -114,24 +107,24 @@ let apply_dyn_fun deflt f lobj =
   f dodecl
 
 let cache_object ((_,lobj) as node) =
-  apply_dyn_fun () (fun d -> d.dyn_cache_function node) lobj
+  apply_dyn_fun (fun d -> d.dyn_cache_function node) lobj
 
 let load_object i ((_,lobj) as node) =
-  apply_dyn_fun () (fun d -> d.dyn_load_function i node) lobj
+  apply_dyn_fun (fun d -> d.dyn_load_function i node) lobj
 
 let open_object i ((_,lobj) as node) =
-  apply_dyn_fun () (fun d -> d.dyn_open_function i node) lobj
+  apply_dyn_fun (fun d -> d.dyn_open_function i node) lobj
 
 let subst_object ((_,lobj) as node) = 
-  apply_dyn_fun lobj (fun d -> d.dyn_subst_function node) lobj
+  apply_dyn_fun (fun d -> d.dyn_subst_function node) lobj
 
 let classify_object lobj =
-  apply_dyn_fun Dispose (fun d -> d.dyn_classify_function lobj) lobj
+  apply_dyn_fun (fun d -> d.dyn_classify_function lobj) lobj
 
 let discharge_object ((_,lobj) as node) =
-  apply_dyn_fun None (fun d -> d.dyn_discharge_function node) lobj
+  apply_dyn_fun (fun d -> d.dyn_discharge_function node) lobj
 
 let rebuild_object lobj =
-  apply_dyn_fun lobj (fun d -> d.dyn_rebuild_function lobj) lobj
+  apply_dyn_fun (fun d -> d.dyn_rebuild_function lobj) lobj
 
 let dump = Dyn.dump
