@@ -9,8 +9,8 @@
 (************************************************************************)
 
 val invfun :
-  Misctypes.quantified_hypothesis ->
-  Globnames.global_reference option ->
+  Tactypes.quantified_hypothesis ->
+  Names.GlobRef.t option ->
   Evar.t Evd.sigma -> Evar.t list Evd.sigma
 val derive_correctness :
   (Evd.evar_map ref ->
