@@ -12,9 +12,9 @@ open Pp
 open CErrors
 open Util
 open Names
-open Term
 open Termops
 open Environ
+open Constr
 open EConstr
 open Vars
 open Namegen
@@ -232,9 +232,8 @@ let inversion_scheme env sigma t sort dep_option inv_op =
   let c = fill_holes pfterm in
   (* warning: side-effect on ownSign *)
   let invProof = it_mkNamedLambda_or_LetIn c !ownSign in
-  let invProof = EConstr.Unsafe.to_constr invProof in
-  let p = Evarutil.nf_evars_universes sigma invProof in
-    p, sigma
+  let p = EConstr.to_constr sigma invProof in
+  p, sigma
 
 let add_inversion_lemma ~poly name env sigma t sort dep inv_op =
   let invProof, sigma = inversion_scheme env sigma t sort dep inv_op in
@@ -252,7 +251,7 @@ let add_inversion_lemma_exn ~poly na com comsort bool tac =
   let env = Global.env () in
   let sigma = Evd.from_env env in
   let sigma, c = Constrintern.interp_type_evars env sigma com in
-  let sigma, sort = Evd.fresh_sort_in_family ~rigid:univ_rigid env sigma comsort in
+  let sigma, sort = Evd.fresh_sort_in_family ~rigid:univ_rigid sigma comsort in
   try
     add_inversion_lemma ~poly na env sigma c sort bool tac
   with
